package us.souther.select;

import java.util.ArrayList;

/**
 * This is the model portion of an MVC app that demonstrates one way to
 * build work with dynamic select boxes.
 * In the real world this bean would most likely access a relational database
 * to get the data.  In order to keep this project entirely contained within
 * a war file, a data structure (HolidayDB) was used instead.
 * The basic concept is the same and this bean should be good enough to use
 * as a starting point in a relational database app.
 * 
 * <b>Legal Notice: </b>
 * <p>
 * This code was written for demonstration purposes only.
 * The author of this code assumes no responsibility for damage resulting in
 * it's use. You may do whatever you like with it but at your own risk.
 * It is your responsibility to understand how it works and determine if it is
 * dependable enough, stable enough, and secure enough for your intended purpose.
 * </p>
 * @author Ben Souther; <a href="mailto:ben@souther.us">ben@souther.us</a>
 * @since  Sat Apr 23 15:56:51 EDT 2005
 */
public class HolidayBean{

    // -----------------------------------------------------------------------
    // ------------------------------------------------------------- Constants
    private static final String[][] MONTHS = new String[][]{
        {"1",  "January"},
        {"2",  "Feburary"},
        {"3",  "March"},
        {"4",  "April"},
        {"5",  "May"},
        {"6",  "June"},
        {"7",  "July"},
        {"8",  "August"},
        {"9",  "September"},
        {"10", "October"},
        {"11", "November"},
        {"12", "December"},
        {"0",  "All"}};

    // -----------------------------------------------------------------------
    // ---------------------------------------------------- Instance Variables
    private HolidayDB db     = null;
    private int       month  = -1;
    private String    key;
    private boolean   showInfo    = false;
    private boolean   formEnabled = false;


    // -----------------------------------------------------------------------
    // ---------------------------------------------------------- Constructors
    /**
     * Creates a new HolidayBean.
     * This is the constructor that will be called when the JSP is first hit.
     * It is necessary to have a zero argument constructor when using the 
     * "class" attribute of a jsp:useBean tag to instanciate the bean 
     * automatically when the JSP first loads.
     */
    public HolidayBean(){
    }


    /**
     * Creates a new HolidayBean.
     * This is the constructor that the servlet (controller) will use when
     * instanciating this bean.
     * @param db The mock database containing the holiday information.
     * @param month The month filter used to restrict the list of holidays.
     *              This value will be passed to the servlet as an HTML form parameter.
     *              It will come from the month select list in on the JSP.
     * @param key The key needed to pull a specific holiday from the database.
     *            This value will be passed to the servlet as an HTML form parameter.
     *            It will come from the holiday select list on the JSP.
     *            If this arg is a blank string we will assume that the 
     *            form submission was triggered by the onclick event of the 
     *            month select list.
     */
    public HolidayBean(HolidayDB db, int month, String key){

        this.db    = db;
        this.month = month;
        this.key   = key;
    }


    // -----------------------------------------------------------------------
    // ------------------------------------------------------ Property methods
    /**
     * Builds a list of Months as a set of HTML Select Option tags.
     * @return the list of months as HTML Select Options.
     * If the month was passed into this bean's constructor, it will
     * appear as a selected month.
     * The bean is capable of building this list without any information
     * from the JSP or Servlet. This method will return a full set of 
     * Option tags, even when the bean has been instanciated with the
     * zero argument constructor.
     */
    public String getMonthsAsOptions(){
        final int INDEX = 0;
        final int NAME  = 1;

        StringBuffer sb = new StringBuffer();
        for(int i = 0; i < MONTHS.length; i++){
            sb.append("<option value=\"");
            sb.append(MONTHS[i][INDEX]);
            sb.append("\"");
            if(this.month != -1 && String.valueOf(this.month).equals(MONTHS[i][INDEX]))
                sb.append(" selected");
            sb.append(">");
            sb.append(MONTHS[i][NAME]);
            sb.append("</option>\n");
        }
        return sb.toString();
    }

    /**
     * Builds a filtered list of holidays as a set of HTML Select Option tags.
     * @return The list of available holidays as HTML Select Option tags.
     * If a key argument passed into this bean's constructor is not empty, it will cause
     * that holiday to appear as selected.
     * The list will be filtered by the month argument passed into the constructor.
     * If this bean was instanciated with the zero argment constructor, this method
     * will return an empty string.
     */
    public String getHolidaysAsOptions(){

        if(null == this.db || -1 == this.month)
            return "";

        //
        // If we were using a relational database right now,
        // we would build the option list by using a query
        // that looks somthing like:
        // "SELECT h.month + '-' = h.day AS key, h.name FROM t_holidays h WHERE MONTH = " 
        // + month + " ORDER BY h.month, h.day;"
        //
        // We would then loop through the ResultSet to build the list of option tags.
        //
        ArrayList holidays = db.selectByMonth(this.month);

        StringBuffer sb = new StringBuffer();
        for(int i = 0; i < holidays.size(); i++){
            Holiday holiday = (Holiday)holidays.get(i);
            sb.append("<option value=\"");
            sb.append(holiday.getKey());
            sb.append("\"");
            if(holiday.getKey().equals(this.key)){
                sb.append(" selected");
                this.formEnabled = true;
            }
            sb.append(">");
            sb.append(holiday.getName());
            sb.append("</option>\n");
        }
        return sb.toString();
    }



    /** 
     * Used to toggle the disabled status of the form's enter button.
     * Read this property from either a jsp:getProperty tag or an EL 
     * expression inside of the HTML enter button.
     * <br />
     * <b>Example:</b>
     * &lt;input type="enter_button" value="Enter" ${holidayBean.isFormEnabled} /&gt;
     * <br />or<br />
     * &lt;input type="enter_button" value="Enter" &lt;jsp:getProperty name="holidayBean" property="isFormEnabled"  /&gt; /&gt;
     * @return "" if the form is ready to be submitted, else " disabled".
     */
    public String getIsFormEnabled(){
        return this.formEnabled ? "" : " disabled";
    }


    /**
     * Shows the date that a selected holiday is celebrated on but only 
     * when the final form has been submitted.
     * The purpose is mainly to demonstrate how to restrict the submitting
     * of a form until all the needed information has been gathered from the
     * select boxes.
     * @return a friendly message telling when this holiday is celebrated.
     */
    public String getInfo(){

        //
        // Make sure all the needed information has been gathered and that
        // the user has specifically submitted the final form.
        //
        if(null == this.db || null == this.key ||
               "".equals(this.key) || !this.formEnabled || !this.showInfo)
            return "";

        Holiday holiday = this.db.getHoliday(this.key);

        return holiday.getName() 
               + " is celebrated on "
               + MONTHS[holiday.getMonth() - 1][1]
               + " "
               + holiday.getDay()
               + ".";
    }



    /**
     * Unless this is specifically set to true, the 
     * getInfo mehod will return an empty string.
     * The purpose is mainly to demonstrate how to 
     * keep a form button disabled until all the needed
     * information has been gathered from the select boxes.
     * @param showInfo form information?
     */
    public void setShowInfo(boolean showInfo){
        this.showInfo = showInfo;
    }

    /**
     * The month as a numeric string.
     * Readonly.
     * @return the month
     */
    public int getMonth(){
        return this.month;
    }

     
    /**
     * This key is also used as the value portion in the
     * option tags return from the getHolidaysAsOptions method.
     * @return The key for retrieving the holiday object.
     */
    public String getKey(){
        return this.key;
    }


    /**
     * For testing and debugging only.
     */
    public static void main(String[] args){
        HolidayBean bean = new HolidayBean();
        System.out.println(bean.getMonthsAsOptions());

        bean = new HolidayBean(new HolidayDB(),0, "12-25");
        System.out.println(bean.getHolidaysAsOptions());
    }
}
