package us.souther.select;

import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;


/**
 * Mock database that returns a list of Holiday Objects.
 * In order to keep this demonstration project entirely contained within
 * a war file, the database had to be mimicked with this object.
 * <br />
 * The point was not to demonstrate how to store data in Java object.
 * I almost left this source file out of the war file because I didn't
 * want people getting too caught up in how it works.  The code that is
 * meant to be studied is in HolidaySelectorServlet.java, HolidayBean.java and
 * the index.jsp page.
 * 
 * <b>Legal Notice:</b>
 * <p>
 * This code was written for demonstration purposes only.
 * The author of this code assumes no responsibility for damage resulting in
 * it's use. You may do whatever you like with it but at your own risk.
 * It is your responsibility to understand how it works and determine if it is
 * dependable enough, stable enough, and secure enough for your intended purpose.
 * </p>
 *
 * @author Ben Souther; <a href="mailto:ben@souther.us">ben@souther.us</a>
 * @since  Sat Apr 23 15:56:51 EDT 2005
 */
public class HolidayDB{

    private HashMap holidays;

    public HolidayDB(){
        holidays = new HashMap();
        addHoliday(1,  1,  "New Year's Day");
        addHoliday(1,  17, "Martin Luther King Jr. Day");
        addHoliday(2,  8,  "Ground Hog Day");
        addHoliday(2,  14, "Valintine's Day");
        addHoliday(2,  22, "Washington's Birthday");
        addHoliday(3,  17, "Evacuation Day");
        addHoliday(4,  18, "Patriot's Day");
        addHoliday(5,  8,  "Mother's Day");
        addHoliday(5,  30, "Memorial Day");
        addHoliday(6,  14, "Flag Day");
        addHoliday(6,  17, "Bunker Hill Day");
        addHoliday(6,  19, "Father's Day");
        addHoliday(7,  4,  "Independence Day");
        addHoliday(9,  5,  "Labor Day");
        addHoliday(10, 3,  "Columbus Day");
        addHoliday(10, 31, "Halloween");
        addHoliday(11, 11, "Veteran's Day");
        addHoliday(11, 24, "Thanksgiving Day");
        addHoliday(12, 24, "Christmas Eve");
        addHoliday(12, 25, "Christmas Day");
    }
    

    public void addHoliday(int    month,
                           int    day,
                           String name){

        Holiday h = new Holiday(month, day, name);
        this.holidays.put(h.getKey(), h);
    }



    /**
     * returns a Holiday object 
     * @param key the key for the holiday object.
     */
    public Holiday getHoliday(Object key){
        return (Holiday)this.holidays.get(key);
    }
    
    /** 
     * @return an array list of Holiday objects sorted by date.
     * @param month use Holiday constants.
     */
    public ArrayList selectByMonth(int month){
        if(month < 0 || month > 12)
            throw new IllegalArgumentException("Invalid month number");
          
        Object[] keys = this.holidays.keySet().toArray();

        ArrayList list = new ArrayList();
        
        for(int i = 0; i < keys.length; i++){
            Holiday h = this.getHoliday(keys[i]);
            if(0 == month  || h.getMonth() == month)
                list.add(h);
        }
        Collections.sort(list);
        return list;

    }


    //
    // for testing and debugging only
    //
    public static void main(String[] args){
        int month; 

        try{
            month = Integer.parseInt(args[0]);
        }catch(Exception e){
            month = 0;
        }

        HolidayDB bean = new HolidayDB();

        ArrayList list = bean.selectByMonth(month);
        for(int i = 0; i < list.size(); i++){
            Holiday holiday = (Holiday)list.get(i);
            System.out.println(holiday.getMonth()
                               + ", " 
                               + holiday.getDay()
                               + " " 
                               + holiday.getName());
        }
    }
}


//
// I didn't make this class public because there is 
// no need for any object other than HolidayDB
// to try to create an object from it.
// 
class Holiday implements Comparable{
    
    // -----------------------------------------------------------------------
    // ------------------------------------------------------------- Constants
    public static final int JANUARY   = 1;
    public static final int FEBRUARY  = 2;
    public static final int MARCH     = 3;
    public static final int APRIL     = 4;
    public static final int MAY       = 5;
    public static final int JUNE      = 6;
    public static final int JULY      = 7;
    public static final int AUGUST    = 8;
    public static final int SEPTEMBER = 9;
    public static final int OCTOBER   = 10;
    public static final int NOVEMBER  = 11;
    public static final int DECEMBER  = 12;


    // -----------------------------------------------------------------------
    // ---------------------------------------------------- Instance Variables
    private String name;
    private int    day;
    private int    month;

    // -----------------------------------------------------------------------
    // ---------------------------------------------------------- Constructors
    public Holiday(int month, int day, String name){
        this.name  = name;
        this.day   = day;
        this.month = month;
    }


    // -----------------------------------------------------------------------
    // ------------------------------------------------------ Property methods
    /**
     * @return The Holiday's name
     */
    public String getName(){
        return this.name;
    }

    //
    // private to keep this property read-only.
    //
    private void setName(String name){
        this.name = name;
    }


    /**
     * @return The day of the month for this holiday.
     */
    public int getDay(){
        return this.day;
    }

    //
    // private to keep this property read-only
    //
    private void setDay(int day){
        this.day = day;
    }

    /**
     * @return The numeric represention for the month that
     *         this holiday is celebrated.
     */
    public int getMonth(){
        return this.month;
    }

    //
    // private to keep this property read-only.
    //
    private void setMonth(int day){
        this.month = month;
    }

    /**
     * An easy way to represent a holiday with a short string.
     * This of course would would present a problem in a 
     * real world app because it makes no provisions for 
     * more than one holiday landing on the same day.
     * Again, this demonstration is about <b>dynamic select lists</b>
     * not about ORMapping.
     * @return numeric month + '-' + numeric day.
     */
    public String getKey(){
        return this.getMonth() + "-" + this.getDay();
    }


    // -----------------------------------------------------------------------
    // ------------------------------------------------------- Utility Methods
    /**
     * required for interface Comparable.
     */
    public int compareTo(Object holiday){

        //
        // Probably not the slickest way to do this but
        // again, this object is only here to mimick what
        // would normally be done by calling a database.
        //
        Holiday h = (Holiday)holiday;
        int thatValue = (h.getMonth() * 33) + h.getDay();
        int thisValue = (this.getMonth() * 33) + this.getDay();

        if(thisValue > thatValue) 
            return 1;

        if(thisValue < thatValue)
            return -1;

        return 0;
    }
}
