package us.souther.select;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import javax.servlet.RequestDispatcher;
import java.io.IOException;

/**
 * This is the controller portion of an MVC app that demonstrates one way to
 * build work with dynamic select boxes.
 *
 * <b>Legal Notice: </b>
 * <p>
 * This code was written for demonstration purposes only.
 * The author of this code assumes no responsibility for damage resulting in
 * it's use. You may do whatever you like with it but at your own risk.
 * It is your responsibility to understand how it works and determine if it is
 * dependable enough, stable enough, and secure enough for your intended purpose.
 * </p>
 * 
 * @author Ben Souther; <a href="mailto:ben@souther.us">ben@souther.us</a>
 * @since  Sat Apr 23 15:56:51 EDT 2005
 */
public class HolidaySelectorServlet extends HttpServlet{

    

    /**
     * Initializes the servlet by instanciating the imatation database
     * object and binding it to context scope.
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        HolidayDB db = new HolidayDB();
        this.getServletContext().setAttribute("db", db);
    }

    /** 
     * Handles all GET requests by simply forwarding the user to the 
     * JSP.
     */ 
    public void doGet(HttpServletRequest request,
                      HttpServletResponse response)
                        throws ServletException, IOException{

        // see the forward method at the bottom of this page.
        forward("index.jsp", request, response);
    }



    /**
     * Handles POST requests from the JSP.
     * Expects the following form parameters.
     * <ol>
     * <li>month:        Must be a numeric string between from 0 to 12;</li>
     * <li>holiday:      {month}-{day}. Example, for Christmas "12-25".
     *                   If null or empty, it is assumed that the user has not
     *                   yet choosen a holiday and that the form was submitted 
     *                   from an onclick event in the months select box.</li>
     * <li>enter_button: If null, we assume that the user has not finshed
     *                   gathering the holiday information before submitting 
     *                   the final form, else we assume that all the other
     *                   parameters are valid and the HolidayBean is given 
     *                   the OK to show the holiday information via the 
     *                   getInfo method.</li>
     * </li>
     * </ol>
     */
    public void doPost(HttpServletRequest request,
                       HttpServletResponse response)
                        throws ServletException, IOException{
        
        //
        // Retrieve and validate the form parameters
        // In the real world, you will probably want to do a little more
        // than blow up with IAEs when these parameters are bad.
        // For the sake of brevity, we'll just blow up if they aren't 
        // what we were looking for.
        //
        String key = request.getParameter("holiday");
        if(null == key)
            throw new IllegalArgumentException("Invalid value in form parameters");
        
        int month;
        try{
           month = Integer.parseInt(request.getParameter("month")); 
           if(month < 0 || month > 12)
               throw new IllegalArgumentException("Invalid value in form parameters");
        }catch(Exception e){
            throw new IllegalArgumentException("Invalid value in form parameters");
        }


        //
        // Get a reference to the database object.
        // We created the DB and bound it to context scope in this
        // servlet's contructor.
        //
        HolidayDB db = (HolidayDB)getServletContext().getAttribute("db");

        //
        // Create the Javabean using the constructor that takes
        // the Database reference, the month and the holiday key
        // as arguments.
        // All the properties in that bean are read-only so this
        // is the only way to set them. This helps to prevent the web 
        // designer from breaking it's functionality by setting 
        // it's properties from a JSP page.
        // 
        // Without using scriptlets, the only way the web designer
        // can access this bean is through it's readonly properties 
        // and it's zero argument constructor.
        //
        HolidayBean hb = new HolidayBean(db, month, key);

        //
        // This is a little absurd.
        // It's only purpose is to provide a way to show something
        // in the JSP after the user has finished gathering holiday 
        // data from the dynamic select lists and has clicked on the
        // form's submit button.  We've disabled the submit button
        // via a property in the HolidayBean until the user has chosen
        // a holiday. Once that is done, they can click the button and 
        // we will allow the bean to show them a message.
        ///
        if(null != request.getParameter("enter_button"))
            hb.setShowInfo(true);
        
        //
        // Bind the bean to request scope so that the JSP can access it.
        // Then, forward the user to the JSP.
        //
        request.setAttribute("holidayBean", hb);
        forward("index.jsp", request, response);
    }
    


    /*
     * private utility for forwarding to JSPs
     * Since we do this more than once in this servlet, it makes sense to 
     * move the functionality into a separate function.
     * Also, checks for the preceeding slash and prepend it if missing.
     */
    private void forward(String              url, 
                         HttpServletRequest  request, 
                         HttpServletResponse response) 
                            throws ServletException, IOException{

        if(!url.startsWith("/"))
            url = "/" + url;
        RequestDispatcher rd = this.getServletContext().getRequestDispatcher(url);
        rd.forward(request, response);
    }
}
