<%-- 
  The point of this demonstration app was to show one way to build use 
  dynamic select lists in a server side java application.  It also serves
  as an example of how the "Model View Controller" design pattern (MVC)
  can effectively move all the branching and looping out of the view tier 
  of an application leaving only HTML and a couple JSP tags.

  The only bean needed for this demonstration is the HolidayBean.
  This bean serves as the model portion of the MVC pattern. It hits a mock 
  database to retrieve a list of holiday objects filtered by the month
  in which they are celebrated.  

  Each time a select list is changed the first form on the page is submitted.
  This is triggered by a small amount of code in the onchange event of each 
  select list. The form always submits to the HolidaySelectorServlet (controller)
  which instanciates both the HolidayBean (model) and the mock database object.

  Upon instanciation, the HolidayBean evaluates the form parameters passed to
  it by the HolidaySelectorServlet and makes a database call to get the holidays
  that meet the criteria. The controller then binds the model to request scope 
  and forwards the page context to the JSP for formatting and presentation.

  One interesting thing to note is that this page uses the jsp:useBean tag to
  find/instanicate the HolidayBean object.  In this tag, we've stated that the
  HolidayBean is to be bound to request scope. Because we've used the class 
  attribute to define the class for this object instead of the type attribute
  the JSP will look for the object in the stated scope (request). If it doesn't 
  find it there, it will instanicate it itself by calling the bean's zero argument
  constructor.  Because, we've provided a zero argument constructor and because
  the bean needs no parameters to build the list of months, we can hit this page
  directly -- bypassing the MVC pattern initially.  You can try this by entering
  http://localhost:8080/DynamicSelectLists/index.jsp in your browser's window.

  If you use http://localhost:8080/DynamicSelectLists the app looks for the default welcome 
  page "index.html", finds that there is a servlet mapping with that name and 
  directs the flow to the doGet method of the HolidaySelectorServlet.
  The servlet then passes the page context to the JSP.  Either way, the initial hit
  is successful.

  This page uses get all of it's dynamic data by reading properties in the 
  HolidayBean via the jsp:getProperty tags.  If you're using JSP 2.0, you could 
  replace these tags with the shorter EL syntax: ${beanName.property}. 
  By building the HTML for the Select Box Options in the JavaBean (model) 
  we've made it possible to remove all the necessary branching and looping required 
  to build them but still give the HTML designer complete control over the look 
  and feel of the application. The designer still has control of the actual 
  SELECT tag and can set any of it's properties, right in this page.



--%>
<jsp:useBean id="holidayBean" class="us.souther.select.HolidayBean" scope="request" />
<html>
  <head>
    <title>
      Dynamic Select Lists
    </title>
  </head>
  <body>
    NOTE: This page requires that your browser has Javascript enabled.
    <br />
    <h2>Dynamic Select Lists</h2>
    Demonstrates one method of building and using dynamic select lists with 
    server side Java using a "Model View Controller" architecture.

    <ol>
      <li>
        Step 1: <b>Choose a month from the first select list.</b>
        <br />
        Notice that the available list of holidays in the second list is updated.
      </li>
      <li> 
        Step 2: <b>Choose a holiday from the second list.</b>
        <br />
        Notice that, once a holiday is choosen the enter button is no longer
        disabled.
      </li>
      <li>
        Step 3: <b>Click on the enter button to view some infomation about the 
        selected holiday.</b>
      </li>
    </ol>
    <br />
    <br />
    <br />
    <form method="post" name="holiday_form" action="HolidaySelector">
      <table width="40%">
        <tr>
          <td width="50%">
            <select name="month" onchange="document.holiday_form.submit()">
              <option value="">Select one</option>
              <jsp:getProperty name="holidayBean" property="monthsAsOptions" />
            </select>
          </td>
          <td width="50%">
            <select name="holiday" onchange="document.holiday_form.submit()">
              <option value="">Select one</option>
               <jsp:getProperty name="holidayBean" property="holidaysAsOptions" />
            </select>
          </td>
        </tr>
      </table>
    </form>
    <form method="post" name="info_form" action="HolidaySelector">
      <input type="hidden" name="month" value="<jsp:getProperty name="holidayBean" property="month" />" />
      <input type="hidden" name="holiday" value="<jsp:getProperty name="holidayBean" property="key" />" />
      <input type="submit" name="enter_button" value="Enter" <jsp:getProperty name="holidayBean" property="isFormEnabled"/> />
      <br />
      <jsp:getProperty name="holidayBean" property="info" />
    <form>
    <br /> <br /> <br /> <br /> <br /> <br /> <br />
    <a href="http://simple.souther.us">Simple Servlet Examples</a>
    <br />
    <a href="doc/index.html">API for this project</a>
  </body>
</html>
